module Hermes
  #Il représente une pièce à fournir et contient
  # un statut (pièce manquante, contrôle en cours, pièce rejetée, pièce valide),
  # une liste des codes erreurs remontés,
  # et le lien vers le ou les fichiers.
  #
  #il est lié à un ou plusieurs applicant, ainsi,
  # une pièce peut être retrouvée et resservir pour plusieurs housingRequestFolder
  class RequiredDocument
    include Hermes::Concerns::CommonFieldsConcern
    
    field :type, type: String #type de piece à fournir, correspond au nom de la famille sur KYC
    field :category, type: String
    field :validity_limit, type: Date #Duréle limite de validité , nil= infini
    field :status, type: String #status de la piece (validée, traitement en cours, erreur)
    field :validation_errors, type: String #liste de erreurs renvoyées par le KYC
    field :tagset_id, type: String #id de l'espace de travail correspondant
    field :tagsetLocation, type: String #url du serveur sur lequel est stokée le document, nil si c'est en interne
    field :doc_ids, type: Array
    field :sne_code, type: String
    field :sne_category, type: String
    field :sne_piece_label, type: String
    field :folder_where_is_required_for_consultation, type: Array
    field :kyc_needed, type: Boolean
    field :folder_where_is_required_for_application, type: Array
    field :last_upload_to_sne_date, type: DateTime
    field :last_download_from_sne_date, type: DateTime
    field :exists_in_sne, type: Boolean
    field :last_xml_validation_report, type: String
    
    belongs_to :applicant, class_name: 'Hermes::Applicant', index: true
    
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    before_save :before_save_action
    after_create :after_create_actions
    after_save :after_save_actions
    after_destroy :after_destroy_actions
    
    def create_tagset
      Tagset.create(is_space: true, title: applicant.full_name + " -- " + type)
    end
    
    def create_doc(filename, blob)
      doc = Doc.create(type: 'file', title: filename, tagset_ids: [self.tagset_id], responsible: user_id)
      doc.blob = blob
      doc.save
      doc
    end
    
    def set_blobs(array_of_blobs)
      #reset du status du document
      
      self.status = "controle en cours"
      self.validation_errors = ""
      self.validity_limit = nil
      self.tagset_id = create_tagset.id
      
      array_of_doc_ids = []
      final_blob = format_to_single_pdf(array_of_blobs)
      doc = create_doc(id.to_s + ".pdf", final_blob)
      array_of_doc_ids.push(doc.id.to_s)
      self.doc_ids = array_of_doc_ids
    end
    
    def format_to_single_pdf(array_of_blobs)
      array_of_pdf_blobs = []
      
      pdf_processor = File::Processor::Actions::Image::Pdf.new
      
      array_of_blobs.each do |item|
        if (item[:filename].split(".")[item[:filename].split(".").length - 1] == "pdf")
          array_of_pdf_blobs.push(item[:blob])
        else
          array_of_pdf_blobs.push(pdf_processor.process(item[:blob]))
        end
      end
      blob_concat = nil
      if (array_of_pdf_blobs.length > 1)
        fp = File::Processor::Actions::Pdf::Concatenate.new
        doc = nil
        
        array_of_pdf_blobs.each do |pdf_blob|
          if doc
            blob_concat = fp.process(pdf_blob)
          else
            doc = create_doc('temporary_blob.pdf', pdf_blob)
            fp.doc_id = doc.id
          end
        end
      else
        blob_concat = array_of_pdf_blobs[0]
      end
      
      blob_concat
    end

=begin
    {
      "auteurDepot" => "TEST SOPRA",
      "categorieCOD" => "SITPRO",
      "categorieLIB" => "Situation professionnelle",
      "creationAuteur" => "TEST SOPRA",
      "creationDAT" => "2020-02-18T19:01:55.000+01:00",
      "dateDepot" => "2020-02-18T19:01:55.000+01:00",
      "derniereActionAuteur" => "TEST SOPRA",
      "derniereActionDAT" => "2020-02-18T19:01:55.000+01:00",
      "derniereActionType" => "Création",
      "documentInterne" => "false",
      "fonctionnelleDAT" => "2034-02-13T00:00:00.000+01:00",
      "idFichier" => "16695",
      "idPiece" => "16695",
      "indicePertinence" => "Non vérifié",
      "nomFichier" => "SNE000185966000025.pdf",
      "numeroUnique" => "0130719003263G3U05",
      "sensibilite" => "false",
      "type" => "Contrat de travail"
    },
=end
    def request_from_sne(parsed_xml_sne_piece, unique_number)
      so = Hermes::BoClient::DownLoadPJTask.new(
        {
          num_unique: unique_number,
          pieceID: parsed_xml_sne_piece["idPiece"]
        }
      )
      puts so.to_xml
      #  if Rails.env == 'test'
      path = [Hermes::Engine.root, %w[spec xml_bouchon], "34.xml"].flatten.join('/').to_s
      pn = Pathname.new(path)
      return {err: 'unknown file'} unless pn.exist?
      return File.read(pn)
      
      #   else
      #    ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml)
      # end
    end
    
    def get_file_from_sne (parsed_xml_sne_piece, unique_number)
      res = Hash.from_xml(request_from_sne(parsed_xml_sne_piece, unique_number).gsub("\n", ""))
      set_blobs([{
                   filename: res["FichierPJ"]["nomFichier"],
                   blob: res["FichierPJ"]["fileContent"]
                 }])
      self.last_download_from_sne_date = DateTime.current
      self.exists_in_sne = true
      save()
    end
    
    def validate_xml(xml, xsd)
      schema_path = [Hermes::Engine.root, %w[spec libs bo_client tasks], xsd].flatten.join('/').to_s
      
      xsd = Nokogiri::XML::Schema(File.read(schema_path))
      xml_validation = xsd.validate(Nokogiri::XML(xml))
      xml_validation.to_s
    end
    
    def create_file_in_sne (unique_number, housing_request_id)
      so = Hermes::BoClient::CreatePJTask.new({
                                                housing_request_id: housing_request_id,
                                                num_unique: unique_number,
                                                required_document_id: id
                                              })
      #puts so.to_xml
      last_xml_validation_report = validate_xml(so.to_xml, "integrationPJ.xsd")
      self.set(exists_in_sne: true, last_upload_to_sne_date: DateTime.current, updated_at: DateTime.current, last_xml_validation_report: last_xml_validation_report)
      so.execute
    end
    
    def update_file_in_sne(unique_number, housing_request_id)
      so = Hermes::BoClient::CreatePJTask.new({
                                                housing_request_id: housing_request_id,
                                                num_unique: unique_number,
                                                required_document_id: id
                                              })
      #puts so.to_xml
      last_xml_validation_report = validate_xml(so.to_xml, "integrationPJ.xsd")
      self.set(last_upload_to_sne_date: DateTime.current, updated_at: DateTime.current, last_xml_validation_report: last_xml_validation_report)
      so.execute
    end
    
    def delete_file_in_sne(unique_number, housing_request_id)
      so = Hermes::BoClient::DeletePJTask.new({
                                                housing_request_id: housing_request_id,
                                                num_unique: unique_number,
                                                required_document_id: id
                                              })
      #puts so.to_xml
      ap validate_xml(so.to_xml, "supprimerPJ.xsd")
      so.execute
    end
    
    def delete_files_in_sne
      
      concerned_unique_numbers_and_housing_request_id.each do |unique_numbers_and_housing_request_id|
        delete_file_in_sne(unique_numbers_and_housing_request_id[:unique_number], unique_numbers_and_housing_request_id[:housing_request_id])
      end
    end
    
    def concerned_unique_numbers_and_housing_request_id
      hrs = HousingRequest.any_of({main_applicant_id: applicant.id}, {main_applicant_id: applicant.id.to_s}, {applicant_ids: applicant.id}, {applicant_ids: applicant.id.to_s})
      unique_numbers = []
      hrs.each do |concerned_hr|
        if concerned_hr.nud_ids || concerned_hr.nud_ids.length == 0
          concerned_hr.nuds.each do |nud|
            unique_numbers.push({unique_number: nud.unique_number, housing_request_id: concerned_hr.id})
          end
        end
      end
      unique_numbers
    end
    
    def upload_file_to_sne
      
      concerned_unique_numbers_and_housing_request_id.each do |unique_numbers_and_housing_request_id|
        if exists_in_sne
          update_file_in_sne(unique_numbers_and_housing_request_id[:unique_number], unique_numbers_and_housing_request_id[:housing_request_id])
        else
          create_file_in_sne(unique_numbers_and_housing_request_id[:unique_number], unique_numbers_and_housing_request_id[:housing_request_id])
        end
      end
    
    end
    
    
    def after_create_actions
    end
    def update_parent_folders_status
      parent_folder_ids = folder_where_is_required_for_consultation + folder_where_is_required_for_application
      parent_folder_ids.each do |folder_id|
        parent_folder = Hermes::HousingRequestFolder.find(folder_id)
        parent_folder.compute_statuses
        parent_folder.save
      end
    end
    
    def set_status
      if !doc_ids || doc_ids.length == 0
        self.status = "Pièce manquante"
      else
        if kyc_needed
          self.status = "Pièce validée"
          self.validation_errors = "[]"
          self.validity_limit = DateTime.current + 3.months
=begin
          if (validation_errors.is_a? String) && (validation_errors != "")
            if validation_errors != "[]"
              self.status = "Pièce invalide"
            else
              self.status = "Pièce validée"
            end
          else
            self.status = "Controle en cours"
          end
=end
        else
          self.status = "Pièce validée"
        end
      end
    end
    def before_save_action
      set_status if validation_errors_changed? || doc_ids_changed? || kyc_needed_changed?
      
    end
    def after_save_actions
      update_parent_folders_status if status_changed?
      upload_file_to_sne if sne_code && sne_category && doc_ids_changed? && !last_download_from_sne_date_changed?
    
    end
    
    def after_destroy_actions
      
      delete_files_in_sne if exists_in_sne
    end
  end

end